* ????-???-???? (CRP-IOL)    DRAWING NO. ????????-???-??
*
*
*
* COMPUTER:  H316, DDP-516
*
*
* PROGRAM CATEGORY:  INPUT/OUTPUT
*
*
* PROGRAM TITLE:  CRP-IOL
*                 CARD READER FORTRAN AND STANDARD INPUT/OUTPUT LIBRARY
*                 USING IOBUS
*
*
*
*
*
*
*
*
*
*
*                            APPROVAL                          DATE
*
*  
*   
*                            PROG----------------------    ------------
*
*
*                            SUPR----------------------    ------------
*
*
*                            QUAL----------------------    ------------
*
*
*                                          NO. OF PAGES
*
*
* AUTHOR   
* 
*    HONEYWELL - COMPUTER CONTROL DIVISION
*
*
* PURPOSE   
*    
*
*    TO PROVIDE A CARD READER FORTRAN AND STANDARD INPUT/OUTPUT
*    LIBRARY USING IOBUS
*
*  
* RESTRICTIONS   
*   
*    THIS LIBRARY WILL WORK ON AN H316 OR DDP-516 COMPUTER WITH A
*    CARD READER. THE EXACT OPTION NUMBER OF THIS CARD READER IS
*    UNKNOWN. IT IS, HOWEVER, NOT THE CARD READER OPTION DOCUMENTED
*    IN THE DDP-516 PROGRAMMERS REFERENCE MANUAL (MODEL 516-61).
*
*    IT MAY WELL BE FOR "CARD READER OPTION 316/516 510X OR THE
*    2410 SERIES CARD READER/PUNCH" AS MENTIONED IN THE
*    "NEW AND UPDATED STANDARD SOFTWARE" NOTE OF JUNE 1971
*
*    SIGNIFICANT DIFFERENCES INCLUDE APPARENTLY DIFFERENT MEANINGS
*    FOR THE SKS CODES. FOR EXAMPLE MODEL 516-61 DOCUMENTS SKS '205
*    AS "SKIP IF NOT END OF FILE" WHERE AS THIS CARD READER OPTION
*    WAITS UNTIL THIS SKIPS BEFORE ISSUING AN OCP TO READ A CARD,
*    WHICH INDICATES IT MUST BE A CONDITION THAT IN NORMAL OPERATION
*    CHANGES STATE JUST ON THE PASSAGE OF TIME. I'VE DECIDED THAT IT
*    IS "SKIP IF NOT BUSY" BUT IT COULD EQUALLY BE SOMETHING LIKE
*    "SKIP IF READY TO FEED A NEW CARD" OR "SKIP IF DONE FEEDING
*    LAST CARD". ANOTHER DIFFERENCE IS THAT A STATUS WORD CAN
*    APPARENTLY BE READ USING AN INA, BY FIRST ISSUING AN OCP '1705.
*
* STORAGE   
*
*    SEE CONTENTS
*
*
* CONTENTS
*
*    MODULE                                              STORAGE   PAGE
*    NAME    DEFINITION                                  OCT  DEC  NO.
*
*    I$CA    CARD READER INPUT SOURCE RECORD
*    I$CBFS  CARD READER BUFFER SIZE                      71   57
*
*    I$CK    CARD READER READ INPUT CHARACTER              7    7
*
*    I$CRDY  CARD READER WAIT UNTIL READY                  7    7
*
*    I$CHER  CARD READER CHECK FOR ERROR                  10    8
*
*    I$CWBS  CARD READER WAIT WHILE BUSY                   4    4
*
*
* USE
*   
*    INTERNAL MODULES  
*
*    NAME    CALLING SEQUENCE              CALLS   ERRORS
*
*    I$CA    CALL  I$CA                    I$CWBS  CE   CARD READER
*            DAC   HEADING BUFFER          I$CRDY        ERROR
*            EOF RETURN                    I$CK
*            NORMAL RETURN                 C$CTLU
*                                          C$CTCI
*
*    I$CBFS  XAC   I$CBFS                          NONE
*    (LINKED WITH I$CA)
*
*    I$CK    CALL  I$CK                    I$CHER  CE   CARD READER
*            ERROR RETURN                                ERROR
*            NORMAL RETURN
*
*            A-REGISTER CONTAINS CHARACTER
*            ON NORMAL RETURN
*
*    I$CRDY  CALL  I$CRDY                  I$CHER  CE   CARD READER
*            ERROR RETURN                                ERROR
*            NORMAL RETURN
*
*    I$CHER  CALL  I$CHER                  C$ER    CE   CARD READER
*            ERROR RETURN                                ERROR
*            NORMAL RETURN
*
*    I$CWBS  CALL  I$CWBS                          NONE
*            RETURN
*
*    EXTERNAL MODULES
*
*    NAME    DEFINITION
*
*    C$CTLU  CHARACTER TABLE LOOK-UP
*
*    C$CTCI  CARD CODE TO ISO CODE CONVERSION TABLE
*
*    C$ER    CONTROL ERROR ROUTINE
*
*
* ERRORS   
*
*    CE    CARD READER ERROR
*
*          THIS ERROR IS REPORTED THROUGH C$ER (CONTROL ERROR
*          ROUTINE).
*
*
* METHOD   
*
*    I$CA READS ONE CARD, CONVERTING THE 6-BIT CHARACTER CODES
*    TO THEIR ISO EQUIVALENTS AND STORING THE RESULT, TWO CHARACTERS
*    PER WORD, INTO THE BUFFER. THE BUFFER SIZE IS NORMALLY 40
*    WORDS (80 CHARACTERS) BUT THIS MAY BE MODIFIED VIA I$CBFS.
*    IF THE BUFFER IS SMALLER THAN 40 WORDS THEN CHARACTER AT THE
*    END OF THE CARD ARE DISCARDED. IF IT IS LARGER THAN 40 WORDS
*    THEN WORDS BEYOND THE SIZE OF THE CARD ARE FILLED WITH BLANK
*    CHARACTERS. IF THE CARD INCLUDES THE ETX CHARACTER, AND THE
*    ISO CONVERSION TABLE INCLUDES A SPACE IN THIS CHARACTER
*    POSITION THEN THE EOF (END OF FILE) RETURN IS TAKEN, ELSE
*    THE NORMAL RETURN.
*
*    I$CK CHECKS THAT THE READER IS OPERATIONAL. IF NOT THEN AN
*    ERROR (CE) IS REPORTED VIA C$ER. IF C$ER RETURNS THEN I$CK
*    WILL RETURN VIA THE ERROR RETURN.
*
*    I$CRDY WAITS FOR THE READER TO BECOME READY, AND CHECKS THAT
*    IT IS OPERATIONAL.
*
*    I$CHER CHECKS THAT THE READER IS OPERATIONAL (I.E. NO ERRORS
*    HAVE OCCURED).
*
*    I$CWBS WAITS UNTIL THE READER IS NOT BUSY.
*
\EJCT
*
* NOTES ON RECOVERY FROM DISASSEMBLY
*
*    THE NAME I$CA IS KNOWN TO BE CORRECT BECAUSE IT IS CALLED FROM
*    THE IOS-O16X DEVICE SELECTION DRIVER OF DAP-16M2, FOR WHICH
*    SOURCE CODE IS AVAILABLE.
*
*    THE NAMES I$CBFS, I$CK, I$CRDY, I$CHER, AND I$CWBS ARE NOT
*    KNOWN, AND IT IS POSSIBLE (EVEN LIKELY) THAT THESE LOCATIONS
*    HAD DIFFERENT NAMES IN THE ORIGINAL SOURCE CODE.
*    THE PORTION OF THE DAP-16M2 SLST FROM WHICH THIS WAS
*    DISASSEMBLED HAS UNUSED WORDS PRECEDING I$CK, I$CRDY, AND
*    I$CHER WHICH, SINCE THESE UNUSED WORDS HAVE ODD LOCATIONS,
*    STRONGLY SUGGESTS THAT THEY ARE SEPARATELY ASSEMBLED SOURCE
*    CODE MODULES.
*
*    THE ROUTINE "ISOC" WITHIN I$CA COULD NOT HAVE BEEN A SEPARATE
*    MODULE BECAUSE IT PRECEDES THE LITERAL POOL FOR I$CA.
*
*
\EJCT
*
*
\SUBR\I$CA\CARD READER INPUT SOURCE RECORD
\SUBR\I$CBFS,BUFS
*
*
\CF5\\H316 AND DDP-516
\REL
*
*
* CARD READER DEVICE CODES
*
* CARD READER ADDRESS
CRDA\EQU\'5\CARD READER ADDRESS
*
* OCP
CRDH\EQU\CRDA\READ ONE HOLLERITH CARD
CRDS\EQU\CRDA+'1700\READ STATUS WORD
*
* INA
CRDI\EQU\CRDA+'1000\CLEAR A AND INPUT CHARACTER
*
*
I$CA\DAC\**
RTRY\LDA*\I$CA\GET BUFFER ADDRESS
\STA\PTRA\PLACE IN POINTER
\CALL\I$CWBS\WAIT UNTIL NOT BUSY
\OCP\CRDH\READ ONE HOLLERITH CARD
\LDX\=0\CLEAR COUNTER
NXTW\JST\ISOC\GET ONE CHARACTER
\LGL\8\PLACE IN UPPER BYTE
\STA*\PTRA\SAVE IN BUFFER
\JST\ISOC\SECOND CHARACTER
\ERA*\PTRA\OR IN FIRST CHAR. OF PAIR
STOR\STA*\PTRA\AND SAVE PAIR
\IRS\PTRA\STEP POINTER
STEP\IRS\0\STEP COUNTER
\LDA\0\GET COUNTER
\CAS\BUFS\COMPARE BUFFER SIZE
\NOP\\PTRA > BUFS
\JMP\BUFF\PTRA = BUFS, SO FULL...
\CAS\=40\COMPARE RECORD SIZE
\NOP\\PTRA > RECS
\JMP\FILL\PTRA = RECS
\JMP\NXTW\GET MORE WORDS
*
* COUNTER BEYOND RECORD SIZE, BUT LESS THAN
* BUFFER SIZE, SO FILL WITH SPACES
FILL\LDA\=A  \TWO SPACES
\JMP\STOR\SAVE CHARACTERS
*
* BUFFER FULL
BUFF\SUB\=40\COMPARE RECORD SIZE
\SMI\\COUNTER > RECORD SIZE?
\JMP\DONE\YES
\JST\ISOC\NO - READ A PAIR
\JST\ISOC\OF CHARACTERS AND DISCARD
\JMP\STEP\STEP COUNTER
*
DONE\CALL\I$CRDY\WAIT UNTIL OK
\JMP\RTRY\ERROR RETURN
\LDX\CCTI\NORMAL RETURN, POINT AT CONV. TABLE
\LDA\'27,1\[(SPACE)ETX,<] IN STANDARD TABLE
\ICL\\LOOK AT TOP BYTE
\LDX\I$CA\PARAMETER ADDRESS
\SUB\='240\SPACE
\SZE\\CONV. TABLE AS EXPECTED?
\JMP\2,1\NO - NORMAL RETURN
\OCP\CRDS\READ STATUS WORD
\INA\CRDI\CLEAR A AND INPUT
\JMP\*-1\WAIT FOR CHARACTER
\SLN\\L.S. BIT NON-ZERO?
\JMP\2,1\NO - NORMAL RETURN
\JMP\1,1\ERROR RETURN
*
*
ISOC\DAC\**\GET NEXT ISO CHARACTER
\CALL\I$CK\GET NEXT CHARACTER
\JMP\RTRY\ERROR RETURN
\CALL\C$CTLU\CHARACTER TABLE LOOKUP
CCTI\XAC\C$CTCI\CARD CODE TO ISO CONVERSION TABLE
\JMP*\ISOC
*
*
* CONSTANTS AND VARIABLES
*
BUFS\DEC\40\BUFFER SIZE
PTRA\BSS\1
*
*
* LITERALS
*
\FIN\\LITERALS
*
*
\END
